@@ -20,11 +20,18 @@ has allowedDomains => ( is => 'rw', isa => 'ArrayRef' );
 has keytab         => ( is => 'rw' );
 has AjaxInitScript => ( is => 'rw', default => '' );
 has Name           => ( is => 'ro', default => 'Kerberos' );
+
+# Override _Ajax InitCmd
 has InitCmd => (
     is      => 'ro',
-    default => q@$self->p->setHiddenFormValue( $req, kerberos => 0, '', 0 )@
+    default =>
+q@$self->p->setHiddenFormValue( $req, kerberos => 0, '', 0 );$self->p->setHiddenFormValue( $req, ajax_auth_token => 0, '', 0 )@
 );
 
+has auth_id => ( is => 'ro', default => 'krb' );
+
+with 'Lemonldap::NG::Portal::Auth::_Ajax';
+
 # INITIALIZATION
 
 sub init {
@@ -48,95 +55,185 @@ sub init {
     return 1;
 }
 
+sub auth_route {
+    my ( $self, $req ) = @_;
+
+    my $auth = $req->env->{HTTP_AUTHORIZATION};
+    if ($auth) {
+        return $self->_handle_ajax_response( $req, $auth );
+    }
+    else {
+        return $self->_request_ajax_credential($req);
+    }
+}
+
 sub extractFormInfo {
     my ( $self, $req ) = @_;
 
+    # This is used when a Combination stack defined different UserDB
+    # for different Kerberos domains
     if ( $req->data->{_krbUser} ) {
         $self->logger->debug(
             'Kerberos ticket already validated for ' . $req->data->{_krbUser} );
         return $self->_checkDomains($req);
     }
 
-    my $auth = $req->env->{HTTP_AUTHORIZATION};
-    unless ($auth) {
-
-        # Case 1: simple usage or first Kerberos Ajax request
-        #         => return 401 to initiate Kerberos
-        if ( !$self->{conf}->{krbByJs} or $req->param('kerberos') ) {
-            $self->logger->debug('Initialize Kerberos dialog');
-
-            # Case 1.1: Ajax request
-            if ( $req->wantJSON ) {
-                $req->response( [
-                        401,
-                        [
-                            'WWW-Authenticate' => 'Negotiate',
-                            'Content-Type'     => 'application/json',
-                            'Content-Length'   => 35
-                        ],
-                        ['{"error":"Authentication required"}']
-                    ]
-                );
-            }
-
-            # Case 1.2: HTML request: display error and initiate Kerberos
-            #           dialog
-            else {
-                $req->error(PE_BADCREDENTIALS);
-                push @{ $req->respHeaders }, 'WWW-Authenticate' => 'Negotiate';
-                my ( $tpl, $prms ) = $self->p->display($req);
-                $req->response(
-                    $self->p->sendHtml(
-                        $req, $tpl,
-                        params => $prms,
-                        code   => 401
-                    )
-                );
-            }
-            return PE_SENDRESPONSE;
+    my $token_id = $req->param('ajax_auth_token');
+    if ($token_id) {
+        my $token = $self->get_auth_token( $req, $token_id );
+        if ( $token->{user} ) {
+            return $self->_krbAuthFinish( $req, $token->{user} );
         }
-
-        # Case 2: Ajax Kerberos request has failed, and javascript has reloaded
-        # page with "kerberos=0". Return an error to be able to switch to
-        # another backend (Combination)
-        # switch to another backend
-        elsif ( defined $req->param('kerberos') ) {
-            $self->userLogger->warn(
-                'Kerberos authentication has failed, back to portal');
-            $self->p->setHiddenFormValue( $req, kerberos => 0, '', 0 );
-            return PE_BADCREDENTIALS;
-        }
-
-        # Case 3: Display kerberos auth page (with javascript)
         else {
-            $self->logger->debug( 'Append ' . $self->Name . ' init/script' );
-
-            # Call kerberos.js if Kerberos is the only Auth module
-            # kerberosChoice.js is used by Choice
-            $self->{AjaxInitScript} =~ s/kerberosChoice/kerberos/;
-
-            # In some Combination scenarios, Kerberos may be called multiple
-            # times but we only want to add the JS once
-            unless ( $req->data->{_krbJsAlreadySent} ) {
-
-                $req->data->{customScript} .= $self->{AjaxInitScript};
-                $self->logger->debug(
-                    "Send init/script -> " . $req->data->{customScript} );
-                $req->data->{_krbJsAlreadySent} = 1;
-            }
-
-            #$self->p->setHiddenFormValue( $req, kerberos => 0, '', 0 );
-            eval( $self->InitCmd );
-            die 'Unable to launch init commmand ' . $self->{InitCmd} if ($@);
-            $req->data->{waitingMessage} = 1;
-            return PE_FIRSTACCESS;
+            return PE_ERROR;
         }
     }
 
+    # Non-AJAX auth
+    my $auth = $req->env->{HTTP_AUTHORIZATION};
+    if ($auth) {
+        return $self->_handle_response( $req, $auth );
+    }
+    else {
+        return $self->_request_credential($req);
+    }
+}
+
+sub _request_credential {
+    my ( $self, $req ) = @_;
+
+    # Case 1: simple usage or first Kerberos Ajax request
+    #         => return 401 to initiate Kerberos
+    if ( !$self->{conf}->{krbByJs} or $req->param('kerberos') ) {
+        $self->logger->debug('Initialize Kerberos dialog');
+
+        # Case 1.1: Ajax request
+        if ( $req->wantJSON ) {
+            $req->response( [
+                    401,
+                    [
+                        'WWW-Authenticate' => 'Negotiate',
+                        'Content-Type'     => 'application/json',
+                        'Content-Length'   => 35
+                    ],
+                    ['{"error":"Authentication required"}']
+                ]
+            );
+        }
+
+        # Case 1.2: HTML request: display error and initiate Kerberos
+        #           dialog
+        else {
+            $req->error(PE_BADCREDENTIALS);
+            push @{ $req->respHeaders }, 'WWW-Authenticate' => 'Negotiate';
+            my ( $tpl, $prms ) = $self->p->display($req);
+            $req->response(
+                $self->p->sendHtml(
+                    $req, $tpl,
+                    params => $prms,
+                    code   => 401
+                )
+            );
+        }
+        return PE_SENDRESPONSE;
+    }
+
+    # Case 2: Ajax Kerberos request has failed, and javascript has reloaded
+    # page with "kerberos=0". Return an error to be able to switch to
+    # another backend (Combination)
+    elsif ( defined $req->param('kerberos') ) {
+        $self->userLogger->warn(
+            'Kerberos authentication has failed, back to portal');
+        $self->p->setHiddenFormValue( $req, kerberos => 0, '', 0 );
+        return PE_BADCREDENTIALS;
+    }
+
+    # Case 3: Display kerberos auth page (with javascript)
+    else {
+        $self->logger->debug( 'Append ' . $self->Name . ' init/script' );
+
+        # Call kerberos.js if Kerberos is the only Auth module
+        # kerberosChoice.js is used by Choice
+        $self->{AjaxInitScript} =~ s/kerberosChoice/kerberos/;
+
+        # In some Combination scenarios, Kerberos may be called multiple
+        # times but we only want to add the JS once
+        unless ( $req->data->{_krbJsAlreadySent} ) {
+
+            $req->data->{customScript} .= $self->{AjaxInitScript};
+            $self->logger->debug(
+                "Send init/script -> " . $req->data->{customScript} );
+            $req->data->{_krbJsAlreadySent} = 1;
+        }
+
+        #$self->p->setHiddenFormValue( $req, kerberos => 0, '', 0 );
+        eval( $self->InitCmd );
+        die 'Unable to launch init commmand ' . $self->{InitCmd} if ($@);
+        $req->data->{waitingMessage} = 1;
+        return PE_FIRSTACCESS;
+    }
+}
+
+sub _handle_response {
+    my ( $self, $req, $auth ) = @_;
+
+    my $client_name = $self->_krb_get_user($auth);
+    if ($client_name) {
+        return $self->_krbAuthFinish( $req, $client_name );
+    }
+    else {
+        return PE_BADCREDENTIALS;
+    }
+}
+
+sub _krbAuthFinish {
+    my ( $self, $req, $client_name ) = @_;
+
+    $self->userLogger->notice("$client_name authentified by Kerberos");
+    $req->data->{_krbUser} = $client_name;
+    if ( $self->conf->{krbRemoveDomain} ) {
+        $client_name =~ s/^(.*)@.*$/$1/;
+    }
+    $req->user($client_name);
+    return $self->_checkDomains($req);
+}
+
+sub _request_ajax_credential {
+    my ( $self, $req ) = @_;
+
+    $self->logger->debug('Initialize Kerberos dialog');
+
+    return [
+        401,
+        [
+            'WWW-Authenticate' => 'Negotiate',
+            'Content-Type'     => 'application/json',
+            'Content-Length'   => 35
+        ],
+        ['{"error":"Authentication required"}']
+    ];
+}
+
+sub _handle_ajax_response {
+    my ( $self, $req, $auth ) = @_;
+
+    my $client_name = $self->_krb_get_user($auth);
+    if ($client_name) {
+        return $self->ajax_success( $req, $client_name );
+    }
+    else {
+        $req->wantErrorRender(1);
+        return $self->p->do( $req, [ sub { PE_BADCREDENTIALS } ] );
+    }
+}
+
+sub _krb_get_user {
+    my ( $self, $auth ) = @_;
+
     # Case 4: an "Authorization header" has been sent
     if ( $auth !~ /^Negotiate (.*)$/ ) {
         $self->userLogger->error('Bad authorization header');
-        return PE_BADCREDENTIALS;
+        return;
     }
 
     # Case 5: Kerberos ticket received
@@ -145,7 +242,7 @@ sub extractFormInfo {
     eval { $data = MIME::Base64::decode($1) };
     if ($@) {
         $self->userLogger->error( 'Bad authorization header: ' . $@ );
-        return PE_BADCREDENTIALS;
+        return;
     }
     $ENV{KRB5_KTNAME} = $self->keytab;
     $self->logger->debug( "Set KRB5_KTNAME env to " . $ENV{KRB5_KTNAME} );
@@ -158,7 +255,7 @@ sub extractFormInfo {
               . 'ticket, make sure the workstation is correctly configured: '
               . 'portal in trusted internet zone, clock synchronization, '
               . 'correct reverse DNS configuration...' );
-        return PE_ERROR;
+        return;
     }
 
     my $status = GSSAPI::Context::accept(
@@ -178,24 +275,18 @@ sub extractFormInfo {
         foreach ( $status->generic_message(), $status->specific_message() ) {
             $self->logger->error($_);
         }
-        return PE_ERROR;
+        return;
     }
     my $client_name;
     $status = $gss_client_name->display($client_name);
-    unless ($status) {
-        $self->logger->error('Unable to display KRB client name');
-        foreach ( $status->generic_message(), $status->specific_message() ) {
-            $self->logger->error($_);
-        }
-        return PE_ERROR;
+    if ($status) {
+        return $client_name;
     }
-    $self->userLogger->notice("$client_name authentified by Kerberos");
-    $req->data->{_krbUser} = $client_name;
-    if ( $self->conf->{krbRemoveDomain} ) {
-        $client_name =~ s/^(.*)@.*$/$1/;
+    $self->logger->error('Unable to display KRB client name');
+    foreach ( $status->generic_message(), $status->specific_message() ) {
+        $self->logger->error($_);
     }
-    $req->user($client_name);
-    return $self->_checkDomains($req);
+    return;
 }
 
 sub _checkDomains {
