@@ -4,14 +4,14 @@ use strict;
 use Mouse;
 use Lemonldap::NG::Portal::Main::Constants;
 
-our $VERSION = '2.0.0';
+our $VERSION = '2.0.12';
 
 extends 'Lemonldap::NG::Portal::Main::Auth';
 
 # INITIALIZATION
 
 sub init {
-    1;
+    return 1;
 }
 
 # RUNNING METHODS
@@ -19,22 +19,22 @@ sub init {
 sub extractFormInfo {
     my ( $self, $req ) = @_;
     $req->user('anonymous');
-    PE_OK;
+    return PE_OK;
 }
 
 sub authenticate {
-    PE_OK;
+    return PE_OK;
 }
 
 sub setAuthSessionInfo {
     my ( $self, $req ) = @_;
-    $req->{sessionInfo}->{'_user'} = 'anonymous';
+    $req->{sessionInfo}->{'_user'}             = 'anonymous';
     $req->{sessionInfo}->{authenticationLevel} = $self->conf->{nullAuthnLevel};
-    PE_OK;
+    return PE_OK;
 }
 
 sub authLogout {
-    PE_OK;
+    return PE_OK;
 }
 
 sub getDisplayType {
