@@ -19,7 +19,7 @@ use Lemonldap::NG::Portal::Main::Constants qw(
   PE_PASSWORDFORMEMPTY
 );
 
-our $VERSION = '2.0.15';
+our $VERSION = '2.0.16';
 
 extends qw(
   Lemonldap::NG::Portal::Main::Auth
@@ -185,4 +185,17 @@ sub setSecurity {
     }
 }
 
+sub getFormParams {
+    my ( $self, $req ) = @_;
+    my $checkLogins = $req->param('checkLogins');
+    $self->logger->debug( $self->prefix . '2f: checkLogins set' )
+      if $checkLogins;
+
+    my $stayConnected = $req->param('stayconnected');
+    $self->logger->debug( $self->prefix . '2f: stayConnected set' )
+      if $stayConnected;
+
+    return ( $checkLogins, $stayConnected );
+}
+
 1;
