@@ -8,8 +8,9 @@ package Lemonldap::NG::Portal::Lib::OpenIDConnect;
 use strict;
 use Crypt::OpenSSL::Bignum;
 use Crypt::OpenSSL::RSA;
+use Crypt::OpenSSL::X509;
 use Digest::SHA
-  qw/hmac_sha256_base64 hmac_sha384_base64 hmac_sha512_base64 sha256 sha384
+  qw/sha1 hmac_sha256_base64 hmac_sha384_base64 hmac_sha512_base64 sha256 sha384
   sha512 sha256_base64 sha384_base64 sha512_base64/;
 use JSON;
 use Lemonldap::NG::Common::FormEncode;
@@ -19,9 +20,12 @@ use Lemonldap::NG::Common::JWT
 use MIME::Base64
   qw/encode_base64 decode_base64 encode_base64url decode_base64url/;
 use Scalar::Util qw/looks_like_number/;
+use URI;
+use URI::QueryParam;
 use Mouse;
+use Crypt::URandom;
 
-use Lemonldap::NG::Portal::Main::Constants qw(PE_OK PE_REDIRECT);
+use Lemonldap::NG::Portal::Main::Constants qw(PE_OK PE_REDIRECT PE_ERROR);
 
 our $VERSION = '2.0.15';
 
@@ -30,12 +34,20 @@ use constant PROFILE => [
     qw/name family_name given_name middle_name nickname preferred_username
       profile picture website gender birthdate zoneinfo locale updated_at/
 ];
-use constant EMAIL   => [qw/email email_verified/];
+use constant EMAIL => [qw/email email_verified/];
 use constant ADDRESS =>
   [qw/formatted street_address locality region postal_code country/];
 use constant PHONE => [qw/phone_number phone_number_verified/];
 
 use constant OIDC_SCOPES => [qw/openid profile email address phone/];
+use constant COMPLEX_CLAIM => {
+    formatted      => "address",
+    street_address => "address",
+    locality       => "address",
+    region         => "address",
+    postal_code    => "address",
+    country        => "address",
+};
 
 # PROPERTIES
 
@@ -58,11 +70,13 @@ has ua => (
     }
 );
 
-has ott => (
+has state_ott => (
     is      => 'rw',
     lazy    => 1,
     default => sub {
         my $ott = $_[0]->{p}->loadModule('::Lib::OneTimeToken');
+        $ott->timeout( $_[0]->conf->{oidcRPStateTimeout}
+              || $_[0]->conf->{timeout} );
         return $ott;
     }
 );
@@ -322,7 +336,7 @@ sub getCallbackUri {
 # @param state State
 # return String Authentication Request URI
 sub buildAuthorizationCodeAuthnRequest {
-    my ( $self, $req, $op, $state ) = @_;
+    my ( $self, $req, $op, $state, $nonce ) = @_;
 
     my $authorize_uri =
       $self->oidcOPList->{$op}->{conf}->{authorization_endpoint};
@@ -339,9 +353,6 @@ sub buildAuthorizationCodeAuthnRequest {
       ->{oidcOPMetaDataOptionsClientID};
     my $scope =
       $self->conf->{oidcOPMetaDataOptions}->{$op}->{oidcOPMetaDataOptionsScope};
-    my $use_nonce =
-      $self->conf->{oidcOPMetaDataOptions}->{$op}
-      ->{oidcOPMetaDataOptionsUseNonce};
     my $response_type = "code";
     my $redirect_uri  = $self->getCallbackUri($req);
     my $display =
@@ -360,9 +371,6 @@ sub buildAuthorizationCodeAuthnRequest {
       $self->conf->{oidcOPMetaDataOptions}->{$op}
       ->{oidcOPMetaDataOptionsAcrValues};
 
-    my $nonce;
-    $nonce = $self->ott->createToken if ($use_nonce);
-
     my $authorize_request_params = {
         response_type => $response_type,
         client_id     => $client_id,
@@ -395,7 +403,119 @@ sub buildAuthorizationCodeAuthnRequest {
     return $authn_uri;
 }
 
+sub isResponseModeAllowed {
+    my ( $self, $flow, $response_mode ) = @_;
+
+    # Query encoding can only be used for authorization code flow
+    # cf oauth-v2-multiple-response-types-1_0.html
+    if ( $response_mode and $response_mode eq "query" ) {
+        return ( $flow eq "authorizationcode" );
+    }
+
+    # Fragment or Form Post are OK for all types
+    # cf oauth-v2-form-post-response-mode-1_0.html
+    return 1;
+}
+
+# Build OpenID Connect response
+# This method does not check if the response mode is allowed for the current
+# grant type. Use isResponseModeAllowed for that
+sub sendOidcResponse {
+    my ( $self, $req, $flow, $response_mode, $redirect_uri, $response_params )
+      = @_;
+
+    $response_mode //= $self->getDefaultResponseModeForFlow($flow);
+
+    if ( $response_mode eq "query" ) {
+        return $self->sendQueryResponse( $req, $redirect_uri,
+            $response_params );
+    }
+    elsif ( $response_mode eq "fragment" ) {
+        return $self->sendFragmentResponse( $req, $redirect_uri,
+            $response_params );
+    }
+    elsif ( $response_mode eq "form_post" ) {
+        return $self->sendFormPostResponse( $req, $redirect_uri,
+            $response_params );
+    }
+    else {
+        $self->logger->error("Unknown response_mode $response_mode");
+        return PE_ERROR;
+    }
+}
+
+sub getDefaultResponseModeForFlow {
+    my ( $self, $flow ) = @_;
+    my %def_flows = (
+        authorizationcode => "query",
+        implicit          => "fragment",
+        hybrid            => "fragment",
+    );
+    return $def_flows{$flow};
+}
+
+sub sendQueryResponse {
+    my ( $self, $req, $redirect_uri, $response_params ) = @_;
+    my $response_uri =
+      $self->getQueryResponse( $redirect_uri, $response_params );
+    return $self->_redirectToUrl( $req, $response_uri );
+}
+
+sub sendFragmentResponse {
+    my ( $self, $req, $redirect_uri, $response_params ) = @_;
+    my $response_uri =
+      $self->getFragmentResponse( $redirect_uri, $response_params );
+    return $self->_redirectToUrl( $req, $response_uri );
+}
+
+sub getQueryResponse {
+    my ( $self, $redirect_uri, $response_params ) = @_;
+    my $uri = URI->new($redirect_uri);
+    for ( keys %$response_params ) {
+        $uri->query_param( $_, $response_params->{$_} );
+    }
+    return $uri;
+}
+
+sub getFragmentResponse {
+    my ( $self, $redirect_uri, $response_params ) = @_;
+    my $uri = URI->new($redirect_uri);
+
+   # Use a temporary URL so we can use QueryParam features to build the fragment
+    my $tmp = URI->new;
+    $tmp->query( $uri->fragment );
+    for ( keys %$response_params ) {
+        $tmp->query_param( $_, $response_params->{$_} );
+    }
+
+    $uri->fragment( $tmp->query );
+
+    return $uri;
+}
+
+sub sendFormPostResponse {
+    my ( $self, $req, $redirect_uri, $response_params ) = @_;
+
+    $self->p->clearHiddenFormValue($req);
+    $req->postUrl($redirect_uri);
+    $req->postFields($response_params);
+    $req->steps( ['autoPost'] );
+    return PE_OK;
+}
+
+sub _redirectToUrl {
+    my ( $self, $req, $response_url ) = @_;
+
+    # We must clear hidden form fields saved from the request (#2085)
+    $self->p->clearHiddenFormValue($req);
+    $self->logger->debug("Redirect user to $response_url");
+    $req->urldc($response_url);
+
+    return PE_REDIRECT;
+}
+
 # Build Authentication Response URI for Authorization Code Flow
+# DEPRECATED, remove in 3.0, use sendOidcResponse instead
 # @param redirect_uri Redirect URI
 # @param code Code
 # @param state State
@@ -404,19 +524,18 @@ sub buildAuthorizationCodeAuthnRequest {
 sub buildAuthorizationCodeAuthnResponse {
     my ( $self, $redirect_uri, $code, $state, $session_state ) = @_;
 
-    my $response_url =
-        $redirect_uri
-      . ( $redirect_uri =~ /\?/ ? '&' : '?' )
-      . build_urlencoded(
-        code => $code,
-        ( $state         ? ( state         => $state )         : () ),
-        ( $session_state ? ( session_state => $session_state ) : () )
-      );
-
-    return $response_url;
+    return $self->getQueryResponse(
+        $redirect_uri,
+        {
+            code => $code,
+            ( $state         ? ( state         => $state )         : () ),
+            ( $session_state ? ( session_state => $session_state ) : () )
+        }
+    );
 }
 
 # Build Authentication Response URI for Implicit Flow
+# DEPRECATED, remove in 3.0, use sendOidcResponse instead
 # @param redirect_uri Redirect URI
 # @param access_token Access token
 # @param id_token ID token
@@ -429,23 +548,25 @@ sub buildImplicitAuthnResponse {
         $state, $session_state, $scope )
       = @_;
 
-    my $response_url = "$redirect_uri#"
-      . build_urlencoded(
-        id_token => $id_token,
-        (
-            $access_token
-            ? ( token_type => 'bearer', access_token => $access_token )
-            : ()
-        ),
-        ( $expires_in    ? ( expires_in    => $expires_in )    : () ),
-        ( $state         ? ( state         => $state )         : () ),
-        ( $scope         ? ( scope         => $scope )         : () ),
-        ( $session_state ? ( session_state => $session_state ) : () )
-      );
-    return $response_url;
+    return $self->getFragmentResponse(
+        $redirect_uri,
+        {
+            id_token => $id_token,
+            (
+                $access_token
+                ? ( token_type => 'bearer', access_token => $access_token )
+                : ()
+            ),
+            ( $expires_in    ? ( expires_in    => $expires_in )    : () ),
+            ( $state         ? ( state         => $state )         : () ),
+            ( $scope         ? ( scope         => $scope )         : () ),
+            ( $session_state ? ( session_state => $session_state ) : () )
+        }
+    );
 }
 
 # Build Authentication Response URI for Hybrid Flow
+# DEPRECATED, remove in 3.0, use sendOidcResponse instead
 # @param redirect_uri Redirect URI
 # @param code Code
 # @param access_token Access token
@@ -460,24 +581,25 @@ sub buildHybridAuthnResponse {
         $expires_in, $state,        $session_state, $scope
     ) = @_;
 
-    my $response_url = "$redirect_uri#"
-      . build_urlencoded(
-        code => $code,
-        (
-            $access_token
-            ? ( token_type => 'bearer', access_token => $access_token )
-            : ()
-        ),
-        (
-            $id_token ? ( id_token => $id_token )
-            : ()
-        ),
-        ( $expires_in    ? ( expires_in    => $expires_in )    : () ),
-        ( $state         ? ( state         => $state )         : () ),
-        ( $scope         ? ( scope         => $scope )         : () ),
-        ( $session_state ? ( session_state => $session_state ) : () )
-      );
-    return $response_url;
+    return $self->getFragmentResponse(
+        $redirect_uri,
+        {
+            code => $code,
+            (
+                $access_token
+                ? ( token_type => 'bearer', access_token => $access_token )
+                : ()
+            ),
+            (
+                $id_token ? ( id_token => $id_token )
+                : ()
+            ),
+            ( $expires_in    ? ( expires_in    => $expires_in )    : () ),
+            ( $state         ? ( state         => $state )         : () ),
+            ( $scope         ? ( scope         => $scope )         : () ),
+            ( $session_state ? ( session_state => $session_state ) : () )
+        }
+    );
 }
 
 sub getAccessTokenFromTokenEndpoint {
@@ -593,7 +715,7 @@ sub checkTokenResponseValidity {
 # Check validity of ID Token
 # return boolean 1 if the token is valid, 0 else
 sub checkIDTokenValidity {
-    my ( $self, $op, $id_token ) = @_;
+    my ( $self, $op, $id_token, $state_nonce ) = @_;
 
     my $client_id =
       $self->conf->{oidcOPMetaDataOptions}->{$op}
@@ -658,15 +780,17 @@ sub checkIDTokenValidity {
 
     # Check nonce
     if ($use_nonce) {
-        my $nonce = $id_token->{nonce};
-        unless ($nonce) {
+        my $id_token_nonce = $id_token->{nonce};
+        unless ($id_token_nonce) {
             $self->logger->error("Nonce was not returned by OP $op");
             return 0;
         }
         else {
             # Get nonce session
-            unless ( $self->ott->getToken($nonce) ) {
-                $self->logger->error("Nonce $nonce verification failed");
+            unless ( $id_token_nonce eq $state_nonce ) {
+                $self->logger->error(
+"Nonce $id_token_nonce verification failed, expected $state_nonce"
+                );
                 return 0;
             }
         }
@@ -1211,24 +1335,16 @@ sub storeState {
     # check if there are data to store
     my $infos;
     foreach (@data) {
-        $infos->{$_}        = $req->{$_}       if $req->{$_};
-        $infos->{"data_$_"} = $req->data->{$_} if $req->data->{$_};
+        $infos->{state}->{$_}        = $req->{$_}       if $req->{$_};
+        $infos->{state}->{"data_$_"} = $req->data->{$_} if $req->data->{$_};
     }
     return unless ($infos);
 
     # Session type
     $infos->{_type} = "state";
 
-    # Set _utime for session autoremove
-    # Use default session timeout and relayState session timeout to compute it
-    my $time         = time();
-    my $timeout      = $self->conf->{timeout};
-    my $stateTimeout = $self->conf->{oidcRPStateTimeout} || $timeout;
-
-    $infos->{_utime} = $time + ( $stateTimeout - $timeout );
-
     # Create state session and store infos
-    return $self->ott->createToken($infos);
+    return $self->state_ott->createToken($infos);
 }
 
 # Extract state information into $self
@@ -1238,16 +1354,15 @@ sub extractState {
     return 0 unless $state;
 
     # Open state session
-    my $stateSession = $self->ott->getToken($state);
+    my $stateSession = $self->state_ott->getToken($state);
 
     return 0 unless $stateSession;
+    return 0 unless $stateSession->{_type} eq "state";
+    return 0 unless $stateSession->{state};
 
     # Push values in $self
-    foreach ( keys %{$stateSession} ) {
-        next
-          if $_ =~
-/^(?:type|_session_id|_session_kind|_utime|tokenTimeoutTimestamp|tokenSessionStartTimestamp)$/;
-        my $tmp = $stateSession->{$_};
+    foreach ( keys %{ $stateSession->{state} } ) {
+        my $tmp = $stateSession->{state}->{$_};
         if (s/^data_//) {
             $req->data->{$_} = $tmp;
         }
@@ -1554,6 +1669,20 @@ sub returnBearerError {
     return $res;
 }
 
+sub invalidClientResponse {
+    my ( $self, $req ) = @_;
+    if ( $req->authorization ) {
+        my ($method) = $req->authorization =~ qw/^(\w+) /;
+        if ($method) {
+            $req->respHeaders( [ 'WWW-Authenticate' => $method ] );
+            return $self->sendOIDCError( $req, 'invalid_client', 401 );
+        }
+    }
+    else {
+        return $self->sendOIDCError( $req, 'invalid_client', 400 );
+    }
+}
+
 sub checkEndPointAuthenticationCredentials {
     my ( $self, $req ) = @_;
 
@@ -1656,13 +1785,19 @@ sub getEndPointAccessToken {
     return $access_token;
 }
 
+# DEPRECATED, remove in 3.0, use getAttributeListFromScopeValue instead
+sub getAttributesListFromClaim {
+    my ( $self, $rp, $scope_value ) = @_;
+    return $self->getAttributesListFromScopeValue( $rp, $scope_value );
+}
+
 # Return list of attributes authorized for a claim
 # @param rp RP name
 # @param claim Claim
 # @return arrayref attributes list
-sub getAttributesListFromClaim {
-    my ( $self, $rp, $claim ) = @_;
-    return $self->rpAttributes->{$rp}->{$claim};
+sub getAttributesListFromScopeValue {
+    my ( $self, $rp, $scope_value ) = @_;
+    return $self->rpAttributes->{$rp}->{$scope_value};
 }
 
 # Return granted scopes for this request
@@ -1758,6 +1893,51 @@ sub buildUserInfoResponse {
         $session->data );
 }
 
+sub _addAttributeToResponse {
+    my ( $self, $req, $data, $userinfo_response, $rp, $attribute ) = @_;
+    my @attrConf = split /;/,
+      ( $self->conf->{oidcRPMetaDataExportedVars}->{$rp}->{$attribute} || "" );
+    my $session_key = $attrConf[0];
+    if ($session_key) {
+        my $type  = $attrConf[1] || 'string';
+        my $array = $attrConf[2] || 'auto';
+
+        my $session_value;
+
+        # Lookup attribute in macros first
+        if ( $self->spMacros->{$rp}->{$session_key} ) {
+            $session_value =
+              $self->spMacros->{$rp}->{$session_key}->( $req, $data );
+
+            # If not found, search in session
+        }
+        else {
+            $session_value = $data->{$session_key};
+        }
+
+        # Handle empty values, arrays, type, etc.
+        $session_value =
+          $self->_formatValue( $session_value, $type, $array,
+            $attribute, $req->user );
+
+        # From this point on, do NOT touch $session_value
+        # or you will break the variable's type.
+
+        # Only release claim if it has a value
+        if ( defined $session_value ) {
+
+            # If this attribute is a standardized subkey (address)
+            if ( COMPLEX_CLAIM->{$attribute} ) {
+                my $superkey = COMPLEX_CLAIM->{$attribute};
+                $userinfo_response->{$superkey}->{$attribute} = $session_value;
+            }
+            else {
+                $userinfo_response->{$attribute} = $session_value;
+            }
+        }
+    }
+}
+
 # Return Hash of UserInfo data
 # @param scope OIDC scope
 # @param rp Internal Relying Party identifier
@@ -1779,54 +1959,30 @@ sub buildUserInfoResponseFromData {
 
     $userinfo_response->{sub} = $user_id;
 
-    # Parse scope and return allowed attributes
-    foreach my $claim ( split( /\s/, $scope ) ) {
-        next if ( $claim eq "openid" );
-        $self->logger->debug("Get attributes linked to claim $claim");
-        my $list = $self->getAttributesListFromClaim( $rp, $claim );
-        next unless $list;
-        foreach my $attribute (@$list) {
-            my @attrConf = split /;/,
-              ( $self->conf->{oidcRPMetaDataExportedVars}->{$rp}->{$attribute}
-                  || "" );
-            my $session_key = $attrConf[0];
-            if ($session_key) {
-                my $type  = $attrConf[1] || 'string';
-                my $array = $attrConf[2] || 'auto';
+    # By default, release all exported attributes
+    if ( $self->conf->{oidcServiceIgnoreScopeForClaims} ) {
+        for my $attribute (
+            keys %{ $self->conf->{oidcRPMetaDataExportedVars}->{$rp} || {} } )
+        {
+            $self->_addAttributeToResponse( $req, $data, $userinfo_response,
+                $rp, $attribute );
+        }
 
-                my $session_value;
-
-                # Lookup attribute in macros first
-                if ( $self->spMacros->{$rp}->{$session_key} ) {
-                    $session_value =
-                      $self->spMacros->{$rp}->{$session_key}->( $req, $data );
-
-                    # If not found, search in session
-                }
-                else {
-                    $session_value = $data->{$session_key};
-                }
-
-                # Handle empty values, arrays, type, etc.
-                $session_value =
-                  $self->_formatValue( $session_value, $type, $array,
-                    $attribute, $req->user );
-
-             # From this point on, do NOT touch $session_value or you will break
-             # the variable's type.
-
-                # Only release claim if it has a value
-                if ( defined $session_value ) {
-
-                    # Address is a JSON object
-                    if ( $claim eq "address" ) {
-                        $userinfo_response->{address}->{$attribute} =
-                          $session_value;
-                    }
-                    else {
-                        $userinfo_response->{$attribute} = $session_value;
-                    }
-                }
+        # Else, iterate through scopes to find allowed attributes
+    }
+    else {
+        foreach my $scope_value ( split( /\s/, $scope ) ) {
+            next if ( $scope_value eq "openid" );
+            $self->logger->debug(
+                "Get attributes linked to scope value $scope_value");
+            my $list =
+              $self->getAttributesListFromScopeValue( $rp, $scope_value );
+            $self->logger->debug(
+                "-> found attributes: " . join( " ", @{ $list || [] } ) );
+            next unless $list;
+            foreach my $attribute (@$list) {
+                $self->_addAttributeToResponse( $req, $data,
+                    $userinfo_response, $rp, $attribute );
             }
         }
     }
@@ -2064,7 +2220,7 @@ sub getIDTokenSub {
 sub key2jwks {
     my ( $self, $key ) = @_;
 
-    my $rsa_pub = Crypt::OpenSSL::RSA->new_public_key($key);
+    my $rsa_pub = Crypt::OpenSSL::RSA->new_private_key($key);
     my @params  = $rsa_pub->get_key_parameters();
 
     return {
@@ -2073,6 +2229,29 @@ sub key2jwks {
     };
 }
 
+# Return X.509 data if public key is a certificate
+# @param key public key or certificate
+# @return HashRef of JWK attributes
+sub getCertInfo {
+    my ( $self, $key ) = @_;
+
+    if ( $key =~ /CERTIFICATE/ ) {
+        my $x509 = Crypt::OpenSSL::X509->new_from_string( $key,
+            Crypt::OpenSSL::X509::FORMAT_PEM );
+        my $der  = $x509->as_string(Crypt::OpenSSL::X509::FORMAT_ASN1);
+        my $hash = sha1($der);
+        return {
+            # Caution, x5c is B64, x5t is B64URL, this is not a mistake
+            x5c => [ encode_base64( $der, '' ) ],
+            x5t => encode_base64url($hash),
+        };
+
+    }
+    else {
+        return {};
+    }
+}
+
 # Build Logout Request URI
 # @param redirect_uri Redirect URI
 # @param id_token_hint ID Token
@@ -2082,9 +2261,9 @@ sub key2jwks {
 sub buildLogoutRequest {
     my ( $self, $redirect_uri, @args ) = @_;
 
-    my @tab = (qw(id_token_hint post_logout_redirect_uri state));
+    my @tab = (qw(id_token_hint post_logout_redirect_uri state client_id));
     my @prms;
-    for ( my $i = 0 ; $i < 3 ; $i++ ) {
+    for ( my $i = 0 ; $i < @tab ; $i++ ) {
         push @prms, $tab[$i], $args[$i]
           if defined( $args[$i] );
     }
@@ -2255,6 +2434,11 @@ sub getUserIDForRP {
       : $data->{$user_id_attribute};
 }
 
+sub generateNonce {
+    my ($self) = @_;
+    return encode_base64url( Crypt::URandom::urandom(16) );
+}
+
 1;
 
 __END__
