@@ -3,15 +3,17 @@ package Lemonldap::NG::Portal::Plugins::Impersonation;
 use strict;
 use Mouse;
 use Lemonldap::NG::Portal::Main::Constants qw(
+  PE_MALFORMEDUSER
   PE_OK PE_BADCREDENTIALS
   PE_IMPERSONATION_SERVICE_NOT_ALLOWED
-  PE_MALFORMEDUSER
 );
 
-our $VERSION = '2.0.9';
+our $VERSION = '2.0.10';
 
-extends 'Lemonldap::NG::Portal::Main::Plugin',
-  'Lemonldap::NG::Portal::Lib::_tokenRule';
+extends qw(
+  Lemonldap::NG::Portal::Main::Plugin
+  Lemonldap::NG::Portal::Lib::_tokenRule
+);
 
 # INITIALIZATION
 
@@ -86,10 +88,10 @@ sub run {
     my $unUser = 0;
     my $loginHistory =
       $req->{sessionInfo}->{_loginHistory};    # Store login history
-    $req->{user} ||= $req->{sessionInfo}->{_impUser};    # If 2FA is enabled
-    my $spoofId = $req->param('spoofId')       # Impersonation required
-      || $req->{sessionInfo}->{_impSpoofId}    # If 2FA is enabled
-      || $req->{user};                         # Impersonation not required
+    $req->{user} ||= $req->{sessionInfo}->{_impUser};   # If 2FA is enabled
+    my $spoofId = $req->param('spoofId')                # Impersonation required
+      || $req->{sessionInfo}->{_impSpoofId}             # If 2FA is enabled
+      || $req->{user};    # Impersonation not required
 
     $self->logger->debug("No impersonation required")
       if ( $spoofId eq $req->{user} );
