@@ -3,12 +3,13 @@ package Lemonldap::NG::Portal::UserDB::OpenIDConnect;
 use strict;
 use Mouse;
 use Lemonldap::NG::Portal::Main::Constants qw(
+  PE_OIDC_AUTH_ERROR
   PE_BADCREDENTIALS
   PE_ERROR
   PE_OK
 );
 
-our $VERSION = '2.0.12';
+our $VERSION = '2.0.15';
 
 extends qw(
   Lemonldap::NG::Common::Module
@@ -26,7 +27,8 @@ sub init {
 
 sub getUser {
     my ( $self, $req ) = @_;
-    my $op = $req->data->{_oidcOPCurrent};
+
+    my ( $op, $access_token ) = $self->getUserInfoParams($req);
 
     # This is likely to happen when running getUser without extractFormInfo
     # see #1980
@@ -35,7 +37,10 @@ sub getUser {
         return PE_ERROR;
     }
 
-    my $access_token = $req->data->{access_token};
+    unless ($access_token) {
+        $self->logger->warn("Could not get Access Token for User Info request");
+        return PE_ERROR;
+    }
 
     my $userinfo_content = $self->getUserInfo( $op, $access_token );
 
@@ -44,6 +49,11 @@ sub getUser {
         return PE_OK;
     }
 
+    # call oidcGotUserInfo hook
+    my $h =
+      $self->p->processHook( $req, 'oidcGotUserInfo', $op, $userinfo_content, );
+    return PE_OIDC_AUTH_ERROR if ( $h != PE_OK );
+
     $req->data->{OpenIDConnect_user_info} = $userinfo_content;
 
     # Check that received sub is the same than current user
